#! /usr/bin/env python2.6

import sys
import pygame

import soaplogic
import soaprenderer

def void():
    pass

class SOAP(object):
    
    def __init__(self, cp_size, sr_filename, levels):
        self.id_manager = soaprenderer.IDManager(sr_filename)
        
        self.levels = levels
        self.current_level = None
        
        self.game = soaplogic.Game()
        self.game.on_finish = self.on_finish_level
        
        self.game_renderer = None
        self.cp_size = cp_size
        
        self.init_default_values()
        
        self.menu = self.create_default_menu()
        self.menu_renderer = self.create_default_menu_renderer()
        
        self.next_level_menu = self.create_next_level_menu()
        self.next_level_menu_renderer = self.create_next_level_menu_renderer()
        
        self.game_over_menu = self.create_game_over_menu()
        self.game_over_menu_renderer = self.create_game_over_menu_renderer()
        
        # reference to the current menu (renderer)
        self.current_menu = self.menu
        self.current_menu_renderer = self.menu_renderer
        self.show_menu = False
        
        # game_renderer gets initialized once a level gets loaded
        self.load_next_level()
        
        self.keybd = pygame.key.get_pressed()
        self.game.player.on_death = self.on_player_death
    
    def on_player_death(self):
        self.show_menu = True
    
    def create_game_over_menu(self):
        menu = soaplogic.Menu()
        menu.add_entry('-> GAME OVER <-', void)
        menu.add_entry('Author:', void)
        menu.add_entry('Syrischer Raubelefant', void)
        menu.add_entry('... aka ...', void)
        menu.add_entry('alex89ru', void)
        menu.add_entry('_______________', void)
        menu.add_entry('Quit', sys.exit)
        return menu
    
    def create_game_over_menu_renderer(self):
        return soaprenderer.MenuRenderer(self.game_over_menu, self.menu_size, 
                                self.text_size, self.text_color, self.bg_color)
    
    def create_next_level_menu(self):
        menu = soaplogic.Menu()
        menu.add_entry('LEVEL COMPLETED', void)
        menu.add_entry('Next level', self.load_next_level)
        menu.add_entry('Quit', sys.exit)
        return menu
    
    def create_next_level_menu_renderer(self):
        return soaprenderer.MenuRenderer(self.next_level_menu, self.menu_size, 
                                self.text_size, self.text_color, self.bg_color)
    
    def on_finish_level(self):
        self.show_menu = True
        self.current_menu = self.next_level_menu
        self.current_menu_renderer = self.next_level_menu_renderer
    
    def reload_current_level(self):
        self.load_level(self.current_level)
    
    def create_game_renderer(self):
        return soaprenderer.GameRenderer(self.cp_size, self.game,
                                        self.id_manager)
    
    def game_over(self): ### TODO
        self.show_menu = True
        self.current_menu = self.game_over_menu
        self.current_menu_renderer = self.game_over_menu_renderer
    
    def load_next_level(self):
        if not self.levels:
            self.game_over()
        else:
            self.show_menu = False
            self.current_menu = self.menu
            self.current_menu_renderer = self.menu_renderer
        
            self.current_level = self.levels.pop(0)
            self.load_level(self.current_level)
    
    def load_level(self, filename):
        self.game.load_level(filename)
        del self.game_renderer
        self.game_renderer = self.create_game_renderer()
    
    def update_keyboard(self):
        self.keybd = pygame.key.get_pressed()
        
        # game has input focus
        if self.show_menu == False:
            for key in self.k_move_up:
                if self.keybd[key]:
                    self.game.move_player(soaplogic.Direction.UP,
                                        self.figure_speed)
            for key in self.k_move_down:
                if self.keybd[key]:
                    self.game.move_player(soaplogic.Direction.DOWN,
                                        self.figure_speed)
            for key in self.k_move_left:
                if self.keybd[key]:
                    self.game.move_player(soaplogic.Direction.LEFT,
                                        self.figure_speed)
            for key in self.k_move_right:
                if self.keybd[key]:
                    self.game.move_player(soaplogic.Direction.RIGHT,
                                        self.figure_speed)
    
    def create_default_menu(self):
        menu = soaplogic.Menu()
        menu.add_entry('Restart level', self.reload_current_level)
        menu.add_entry('Hide menu', self.toggle_menu)
        menu.add_entry('Quit', sys.exit)
        return menu
    
    def create_default_menu_renderer(self):
        return soaprenderer.MenuRenderer(self.menu, self.menu_size, 
                                self.text_size, self.text_color, self.bg_color)
    
    def init_default_keys(self):
        # using WASD / arrow keys for navigation
        self.k_move_up = [pygame.K_w, pygame.K_UP]
        self.k_move_down = [pygame.K_s, pygame.K_DOWN]
        self.k_move_left = [pygame.K_a, pygame.K_LEFT]
        self.k_move_right = [pygame.K_d, pygame.K_RIGHT]
        
        # confirming inside menu via [RETURN] and [ENTER]
        self.k_confirm = [pygame.K_RETURN, pygame.K_KP_ENTER]
        
        # open/close menu via [ESC] [F1] and [M]
        self.k_toggle_menu = [pygame.K_ESCAPE, pygame.K_F1, pygame.K_m]
        
        # for fast termination press [F10]
        self.k_quit = [pygame.K_F10]
    
    def init_default_values(self):
        self.figure_speed = 150.0
        
        # menu stuff
        self.text_color = (255, 255, 0)
        self.bg_color = (0, 0, 30, 200)
        self.text_size = 40
        cp_w, cp_h = self.cp_size 
        self.menu_size = (cp_w * 0.75, cp_h * 0.75)
        
        self.init_default_keys()
    
    def toggle_menu(self):
        self.show_menu = not self.show_menu
    
    def update_events(self):
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN:
                if event.key in self.k_quit:
                    sys.exit()
                elif event.key in self.k_toggle_menu:
                    self.toggle_menu()
                
                # menu has input focus
                elif self.show_menu:
                    if event.key in self.k_move_up:
                        self.current_menu.previous_entry()
                    elif event.key in self.k_move_down:
                        self.current_menu.next_entry()
                    elif event.key in self.k_confirm:
                        self.current_menu.execute()
    
    def update(self):
        self.update_events()
        self.update_keyboard()
        self.game.update()
        self.game_renderer.update()
        
    def render(self, dest, pos):
        self.game_renderer.render(dest, pos)
        
        if self.show_menu:
            self.current_menu_renderer.render(dest)

def main():
    pygame.init()
    
    icon = pygame.image.load('graphics/icon.png')
    pygame.display.set_icon(icon)
    
    w, h = 640, 480
    screen = pygame.display.set_mode((w, h))
    
    title = 'SOAP - sun of a peach'
    pygame.display.set_caption(title)
    
    levels = ['data/level1.sl', 'data/level2.sl', 'data/level3.sl', 'data/level4.sl']
    soap = SOAP((620, 460), 'data/rp.sr', levels)
    
    render_pos = (10, 10)
    
    while True:
        soap.update()
        soap.render(screen, render_pos)
        pygame.display.flip()
    
    pygame.quit()

if __name__ == '__main__':
    main()
