#! /usr/bin/env python2.6

import sys
import threading
import copy
import pygame

import soaplogic
import soaprenderer

def move_player(map, fig, dir, speed):
    if map.aligend_to_tilegrid(fig.pos): # covers a tile 
        i_x, i_y = map.coords_to_tile_index(fig.pos)
        if dir == soaplogic.Direction.LEFT:
            i_x -= 1
        elif dir == soaplogic.Direction.RIGHT:
            i_x += 1
        elif dir == soaplogic.Direction.UP:
            i_y -= 1
        elif dir == soaplogic.Direction.DOWN:
            i_y += 1
        else:
            errmsg('move_figure: invalid direction')
        dest_tile_pos = (i_x, i_y)
        
        if map.is_valid_index(dest_tile_pos):
            dest = map.tile_index_to_coords(dest_tile_pos)
            fig.go_to(dest, speed)


def save_map_to_file(filename, map):
    MAGIC = soaplogic.TileMapLoader.MAGIC
    w, h = map.size
    t_w, t_h = map.tile_size
    
    lines = []
    lines.append(MAGIC)
    
    size_list = [str(w), str(h), str(t_w), str(t_h)]
    size_str = ' '.join(size_list)
    lines.append(size_str)
    
    for row in map.tiles:
        line_buf = []
        for tile in row:
            # defaults to not solid
            s = '_'
            if tile.solid:
                s = '#'
            
            entry = '{id}|{solid}'.format(id = tile.id, solid = s)
            line_buf.append(entry)
        
        line_buf_str = ' '.join(line_buf)
        lines.append(line_buf_str)
    
    final_output = '\n'.join(lines)
    f = open(filename, 'w')
    f.write(final_output)
    f.close()

current_tile = soaplogic.Tile()


def same_tile(t1, t2):
    return t1.id == t2.id and t1.solid == t2.solid

def main():
    fill_from = None
    fill_to = None
    pygame.init()
    
    w, h = 1024, 768
    screen = pygame.display.set_mode((w, h))
    
    cp_size = (w, h)
    
    map_size = (50, 50)
    tile_size = (32, 32)
    
    id_man = soaprenderer.IDManager('data/rp.sr')
    
    player_id = 'player0'
    player = soaplogic.Figure()
    speed = 200.0
    
    game = soaplogic.Game()
    #game.create_empty_map(map_size, tile_size)
    game.load_map('data/map4.sm')
    game.register_player(player, player_id, (0, 0))
    
    game_renderer = soaprenderer.GameRenderer(cp_size, game, id_man)

    def prompt_thread():
        global current_tile
        while 1:
            player_pos = game.map.coords_to_tile_index(game.player.pos)
            print 'player_pos', player_pos
            line = raw_input()
            words = line.split()
            # action argument
            player_pos = game.map.coords_to_tile_index(game.player.pos)
            if len(words) == 1:
                action = words.pop(0)
                if action == 'get':
                    current_tile = copy.deepcopy(game.map.get_tile(player_pos))
                    print 'get ->', current_tile.id, current_tile.solid
            if len(words) == 2:
                action = words.pop(0)
                # one more argument left
                if action == 'save':
                    filename = words.pop(0)
                    save_map_to_file(filename, game.map)
                    print '* saving done *'
            elif len(words) == 3:
                action = words.pop(0)
                if action == 'set':
                    id = words.pop(0)
                    s = words.pop(0)
                    if id not in id_man.img_dict:
                        print id, 'not found'
                    else:
                        current_tile.id = id
                        if s == '#':
                            current_tile.solid = True
                        else: # s == '_'
                            current_tile.solid = False
            else:
                print 15*'-'
    
    thread = threading.Thread()
    thread.run = prompt_thread
    thread.start()
    
    running = True
    while running:
        cur_tile_pos = game.map.coords_to_tile_index(game.player.pos)
        print cur_tile_pos
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_F1:
                    fill_from = cur_tile_pos
                elif event.key == pygame.K_F2:
                    fill_to = cur_tile_pos
                elif event.key == pygame.K_F3:
                    fill_from = None
                    fill_to = None
        
        keybd = pygame.key.get_pressed()
        #print game.map.coords_to_tile_index(player.pos)
        if keybd[pygame.K_w]:
            move_player(game.map, player, soaplogic.Direction.UP, speed)
        if keybd[pygame.K_a]:
            move_player(game.map, player, soaplogic.Direction.LEFT, speed)
        if keybd[pygame.K_s]:
            move_player(game.map, player, soaplogic.Direction.DOWN, speed)
        if keybd[pygame.K_d]:
            move_player(game.map, player, soaplogic.Direction.RIGHT, speed)
        
        
        
        if keybd[pygame.K_RETURN]:
            if fill_from is None:
                game.map.set_tile_prop(cur_tile_pos, current_tile.id, current_tile.solid)
                game_renderer.bg.original_bg = game_renderer.bg.create_background(game.map)
            elif fill_to is not None:
                f_x, f_y = fill_from
                t_x, t_y = fill_to
                for x in range(f_x, t_x + 1):
                    for y in range(f_y, t_y + 1):
                        game.map.set_tile_prop((x, y), current_tile.id, current_tile.solid)
                game_renderer.bg.original_bg = game_renderer.bg.create_background(game.map)
                fill_from = None
                fill_to = None
        
        game.update()
        game_renderer.update()
        
        game_renderer.render(screen)
        
        pygame.display.flip()
    
    pygame.quit()

if __name__ == '__main__':
    main()
